@@ -1,38 +1,47 @@
-use crate::types::{
-    Camera, DirectionalLight, DirectionalLightChange, DirectionalLightHandle, Material, MaterialChange, Mesh, Object,
-    RawMaterialHandle, RawObjectHandle, Texture,
+use crate::{
+    resources::{MaterialManager, ObjectManager, TextureManager},
+    types::{Camera, DirectionalLight, DirectionalLightChange, DirectionalLightHandle, Mesh, Object, RawObjectHandle},
+    RendererMode,
 };
 use glam::Mat4;
 use parking_lot::Mutex;
-use rend3_types::{
-    MaterialHandle, MeshHandle, ObjectHandle, RawDirectionalLightHandle, TextureFromTexture, TextureHandle,
-};
+use rend3_types::{MaterialHandle, MeshHandle, ObjectHandle, RawDirectionalLightHandle, TextureHandle};
 use std::mem;
+use wgpu::{CommandBuffer, Device, Texture, TextureDescriptor, TextureView};
 
+#[allow(clippy::type_complexity)]
 pub enum Instruction {
     AddMesh {
         handle: MeshHandle,
         mesh: Mesh,
     },
-    AddTexture2D {
-        handle: TextureHandle,
-        texture: Texture,
-    },
-    AddTexture2DFromTexture {
-        handle: TextureHandle,
-        texture: TextureFromTexture,
-    },
-    AddTextureCube {
+    AddTexture {
         handle: TextureHandle,
+        desc: TextureDescriptor<'static>,
         texture: Texture,
+        view: TextureView,
+        buffer: Option<CommandBuffer>,
+        cube: bool,
     },
     AddMaterial {
         handle: MaterialHandle,
-        material: Material,
+        fill_invoke: Box<
+            dyn FnOnce(&mut MaterialManager, &Device, RendererMode, &mut TextureManager, &MaterialHandle) + Send + Sync,
+        >,
     },
     ChangeMaterial {
-        handle: RawMaterialHandle,
-        change: MaterialChange,
+        handle: MaterialHandle,
+        change_invoke: Box<
+            dyn FnOnce(
+                    &mut MaterialManager,
+                    &Device,
+                    RendererMode,
+                    &mut TextureManager,
+                    &mut ObjectManager,
+                    &MaterialHandle,
+                ) + Send
+                + Sync,
+        >,
     },
     AddObject {
         handle: ObjectHandle,
