@@ -1,9 +1,6 @@
 use crate::{
     instruction::InstructionStreamPair,
-    resources::{
-        CameraManager, DirectionalLightManager, MaterialManager, MeshManager, ObjectManager, TextureManager,
-        STARTING_2D_TEXTURES, STARTING_CUBE_TEXTURES,
-    },
+    resources::{CameraManager, DirectionalLightManager, MaterialManager, MeshManager, ObjectManager, TextureManager},
     util::mipmap::MipmapGenerator,
     InstanceAdapterDevice, Renderer, RendererInitializationError,
 };
@@ -16,20 +13,23 @@ pub fn create_renderer(
     iad: InstanceAdapterDevice,
     aspect_ratio: Option<f32>,
 ) -> Result<Arc<Renderer>, RendererInitializationError> {
+    profiling::scope!("Renderer::new");
+
     let features = iad.device.features();
+    let limits = iad.device.limits();
 
     let camera_manager = RwLock::new(CameraManager::new(Camera::default(), aspect_ratio));
 
     let texture_manager_2d = RwLock::new(TextureManager::new(
         &iad.device,
         iad.mode,
-        STARTING_2D_TEXTURES,
+        limits.max_sampled_textures_per_shader_stage,
         TextureViewDimension::D2,
     ));
     let texture_manager_cube = RwLock::new(TextureManager::new(
         &iad.device,
         iad.mode,
-        STARTING_CUBE_TEXTURES,
+        limits.max_sampled_textures_per_shader_stage,
         TextureViewDimension::Cube,
     ));
     let mesh_manager = RwLock::new(MeshManager::new(&iad.device));
@@ -66,7 +66,7 @@ pub fn create_renderer(
         object_manager,
         directional_light_manager,
 
-        mipmap_generator: Mutex::new(mipmap_generator),
+        mipmap_generator,
         profiler: Mutex::new(profiler),
     }))
 }
