@@ -5,7 +5,7 @@ use crate::{
     INTERNAL_SHADOW_DEPTH_FORMAT, SHADOW_DIMENSIONS,
 };
 use arrayvec::ArrayVec;
-use glam::{Mat4, Vec3A};
+use glam::{Mat4, Vec3, Vec3A};
 use rend3_types::{DirectionalLightChange, RawDirectionalLightHandle};
 use std::{
     mem::{self, size_of},
@@ -57,6 +57,8 @@ pub struct DirectionalLightManager {
 }
 impl DirectionalLightManager {
     pub fn new(device: &Device) -> Self {
+        profiling::scope!("DirectionalLightManager::new");
+
         let registry = ResourceRegistry::new();
 
         let buffer = WrappedPotBuffer::new(
@@ -167,13 +169,25 @@ impl DirectionalLightManager {
 fn shadow(l: &InternalDirectionalLight, user_camera: &CameraManager) -> ArrayVec<CameraManager, 4> {
     let mut cascades = ArrayVec::new();
 
+    let camera_location = user_camera.location();
+
+    let shadow_texel_size = l.inner.distance / SHADOW_DIMENSIONS as f32;
+
+    let origin_view = Mat4::look_at_lh(Vec3::ZERO, l.inner.direction, Vec3::Y);
+    let camera_origin_view = origin_view.transform_point3(camera_location);
+
+    let offset = camera_origin_view.truncate() % shadow_texel_size;
+    let shadow_location = camera_origin_view - Vec3::from((offset, 0.0));
+
+    let inv_origin_view = origin_view.inverse();
+    let new_shadow_location = inv_origin_view.transform_point3(shadow_location);
+
     cascades.push(CameraManager::new(
         Camera {
             projection: CameraProjection::Orthographic {
                 size: Vec3A::splat(l.inner.distance),
-                direction: l.inner.direction.into(),
             },
-            location: user_camera.get_data().location,
+            view: Mat4::look_at_lh(new_shadow_location, new_shadow_location + l.inner.direction, Vec3::Y),
         },
         None,
     ));
@@ -245,6 +259,8 @@ fn vec_min_max(iter: impl IntoIterator<Item = Vec3A>) -> (Vec3A, Vec3A) {
 }
 
 fn create_shadow_texture(device: &Device, count: u32) -> (TextureView, Vec<Arc<TextureView>>) {
+    profiling::scope!("shadow texture creation");
+
     let texture = device.create_texture(&TextureDescriptor {
         label: Some("shadow texture"),
         size: Extent3d {
@@ -289,6 +305,7 @@ fn create_shadow_texture(device: &Device, count: u32) -> (TextureView, Vec<Arc<T
 }
 
 fn create_shadow_bgl(device: &Device) -> BindGroupLayout {
+    profiling::scope!("shadow bgl creation");
     device.create_bind_group_layout(&BindGroupLayoutDescriptor {
         label: Some("shadow bgl"),
         entries: &[
@@ -320,6 +337,7 @@ fn create_shadow_bgl(device: &Device) -> BindGroupLayout {
 }
 
 fn create_shadow_bg(device: &Device, bgl: &BindGroupLayout, buffer: &Buffer, view: &TextureView) -> BindGroup {
+    profiling::scope!("shadow bg creation");
     BindGroupBuilder::new(Some("shadow bg"))
         .with_buffer(buffer)
         .with_texture_view(view)
