@@ -42,7 +42,8 @@ pub struct MeshBuffers {
     pub vertex_position: Buffer,
     pub vertex_normal: Buffer,
     pub vertex_tangent: Buffer,
-    pub vertex_uv: Buffer,
+    pub vertex_uv0: Buffer,
+    pub vertex_uv1: Buffer,
     pub vertex_color: Buffer,
     pub vertex_mat_index: Buffer,
 
@@ -54,9 +55,10 @@ impl MeshBuffers {
         rpass.set_vertex_buffer(0, self.vertex_position.slice(..));
         rpass.set_vertex_buffer(1, self.vertex_normal.slice(..));
         rpass.set_vertex_buffer(2, self.vertex_tangent.slice(..));
-        rpass.set_vertex_buffer(3, self.vertex_uv.slice(..));
-        rpass.set_vertex_buffer(4, self.vertex_color.slice(..));
-        rpass.set_vertex_buffer(5, self.vertex_mat_index.slice(..));
+        rpass.set_vertex_buffer(3, self.vertex_uv0.slice(..));
+        rpass.set_vertex_buffer(4, self.vertex_uv1.slice(..));
+        rpass.set_vertex_buffer(5, self.vertex_color.slice(..));
+        rpass.set_vertex_buffer(6, self.vertex_mat_index.slice(..));
         rpass.set_index_buffer(self.index.slice(..), IndexFormat::Uint32);
     }
 }
@@ -76,6 +78,8 @@ pub struct MeshManager {
 
 impl MeshManager {
     pub fn new(device: &Device) -> Self {
+        profiling::scope!("MeshManager::new");
+
         let buffers = create_buffers(device, STARTING_VERTICES, STARTING_INDICES);
 
         let vertex_count = STARTING_VERTICES;
@@ -108,6 +112,8 @@ impl MeshManager {
         handle: &MeshHandle,
         mesh: Mesh,
     ) {
+        profiling::scope!("MeshManager::fill");
+
         assert!(mesh.validate());
 
         let vertex_count = mesh.vertex_positions.len();
@@ -148,9 +154,14 @@ impl MeshManager {
             bytemuck::cast_slice(&mesh.vertex_normals),
         );
         queue.write_buffer(
-            &self.buffers.vertex_uv,
+            &self.buffers.vertex_uv0,
             (vertex_range.start * VERTEX_UV_SIZE) as BufferAddress,
-            bytemuck::cast_slice(&mesh.vertex_uvs),
+            bytemuck::cast_slice(&mesh.vertex_uv0),
+        );
+        queue.write_buffer(
+            &self.buffers.vertex_uv1,
+            (vertex_range.start * VERTEX_UV_SIZE) as BufferAddress,
+            bytemuck::cast_slice(&mesh.vertex_uv1),
         );
         queue.write_buffer(
             &self.buffers.vertex_color,
@@ -188,7 +199,7 @@ impl MeshManager {
     }
 
     pub fn ready(&mut self) {
-        profiling::scope!("Mesh Manager Ready");
+        profiling::scope!("MeshManager::ready");
 
         let vertex_alloc = &mut self.vertex_alloc;
         let index_alloc = &mut self.index_alloc;
@@ -205,6 +216,8 @@ impl MeshManager {
         needed_verts: u32,
         needed_indices: u32,
     ) {
+        profiling::scope!("reallocate mesh buffers");
+
         let new_vert_count = (self.vertex_count + needed_verts as usize).next_power_of_two();
         let new_index_count = (self.index_count + needed_indices as usize).next_power_of_two();
 
@@ -254,8 +267,16 @@ impl MeshManager {
             );
             copy_vert(
                 encoder,
-                &self.buffers.vertex_uv,
-                &new_buffers.vertex_uv,
+                &self.buffers.vertex_uv0,
+                &new_buffers.vertex_uv0,
+                mesh,
+                &new_vert_range,
+                VERTEX_UV_SIZE,
+            );
+            copy_vert(
+                encoder,
+                &self.buffers.vertex_uv1,
+                &new_buffers.vertex_uv1,
                 mesh,
                 &new_vert_range,
                 VERTEX_UV_SIZE,
@@ -322,6 +343,8 @@ fn copy_vert(
 }
 
 fn create_buffers(device: &Device, vertex_count: usize, index_count: usize) -> MeshBuffers {
+    profiling::scope!("mesh buffers creation");
+
     let position_bytes = vertex_count * VERTEX_POSITION_SIZE;
     let normal_bytes = vertex_count * VERTEX_NORMAL_SIZE;
     let tangent_bytes = vertex_count * VERTEX_TANGENT_SIZE;
@@ -351,8 +374,15 @@ fn create_buffers(device: &Device, vertex_count: usize, index_count: usize) -> M
         mapped_at_creation: false,
     });
 
-    let vertex_uv = device.create_buffer(&BufferDescriptor {
-        label: Some("uv vertex buffer"),
+    let vertex_uv0 = device.create_buffer(&BufferDescriptor {
+        label: Some("uv0 vertex buffer"),
+        size: uv_bytes as BufferAddress,
+        usage: BufferUsages::COPY_SRC | BufferUsages::COPY_DST | BufferUsages::VERTEX | BufferUsages::STORAGE,
+        mapped_at_creation: false,
+    });
+
+    let vertex_uv1 = device.create_buffer(&BufferDescriptor {
+        label: Some("uv1 vertex buffer"),
         size: uv_bytes as BufferAddress,
         usage: BufferUsages::COPY_SRC | BufferUsages::COPY_DST | BufferUsages::VERTEX | BufferUsages::STORAGE,
         mapped_at_creation: false,
@@ -383,7 +413,8 @@ fn create_buffers(device: &Device, vertex_count: usize, index_count: usize) -> M
         vertex_position,
         vertex_normal,
         vertex_tangent,
-        vertex_uv,
+        vertex_uv0,
+        vertex_uv1,
         vertex_color,
         vertex_mat_index,
         index,
